#
#  DeviceLabeler
#
#  $Id: plugin.py,v 1.0 2011/03/04 23:17:00 dre Exp $
#
#  Coded by dre (c) 2010
#  Coding idea and design by dre
#  Support: www.dreambox-tools.info
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from Components.config import config, ConfigText, NoSave, getConfigListEntry
from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText
from Components.Pixmap import Pixmap
from Plugins.Plugin import PluginDescriptor
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from enigma import RT_HALIGN_LEFT, RT_HALIGN_RIGHT, eListboxPythonMultiContent, gFont, eConsoleAppContainer
import os

def isInstalled(package):
	package = 'Package: ' + package
	try:
		f = open('/usr/lib/opkg/status', 'r')
		for line in f:
			if line.strip() == package:
				f.close()
				return True
	except IOError:
		pass

	return False

def main(session,**kwargs):
	if isInstalled('e2fsprogs-tune2fs') == False:
		session.open(MessageBox, "tune2fs is not installed but is required to run the plugin", MessageBox.TYPE_ERROR, timeout=5)
	else:
		session.open(DeviceLabelerScreen)
	
class deviceMenuList(MenuList):
	def __init__(self, list):
		MenuList.__init__(self, list, False, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 22))
		self.l.setFont(1, gFont("Regular", 18))
		self.l.setItemHeight(130)
		
def deviceMenuListEntry(device, size,currentlabel):
	res = [ (device, size,currentlabel) ]
	res.append(MultiContentEntryText(pos=(230,15),size=(370,25), font=0, text=device))
	res.append(MultiContentEntryText(pos=(230,60),size=(370,25), font=0, text=size))
	res.append(MultiContentEntryText(pos=(230,90),size=(370,25), font=0, text=currentlabel))
	res.append(MultiContentEntryText(pos=(0,17),size=(200,25), font=1, flags=RT_HALIGN_RIGHT, text="Device:"))
	res.append(MultiContentEntryText(pos=(0,62),size=(200,25), font=1, flags=RT_HALIGN_RIGHT, text="Size:"))
	res.append(MultiContentEntryText(pos=(0,92),size=(200,25), font=1, flags=RT_HALIGN_RIGHT, text="Current label:"))

	return res
	
class DeviceLabelerScreen(Screen, HelpableScreen):
	skin = """
		<screen position="center,center" size="600,275" title="Device Labeler" >
			<widget name="entryinfo" position="500,0" size="100,30" halign="right" font="Regular;18" transparent="1" />
			<widget name="menulist" position="0,40" size="600,130" scrollbarMode="showNever"/>
			<widget name="changeButton" position="275,211" size="200,25" font="Regular;18" foregroundColor="#f0f0f0" transparent="1"/>
			<eLabel foregroundColor="#f0f0f0" font="Regular;18" position="275,236" size="200,25" text="Cancel" transparent="1"/>
			<ePixmap alphatest="on" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/DeviceLabeler/DeviceLabeler.png" position="70,214" size="100,40"/>
			<widget name="changeButtonPixmap" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/DeviceLabeler/ok.png" position="230,210" size="35,25" alphatest="on"/>
			<ePixmap alphatest="on" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/DeviceLabeler/exit.png" position="230,235" size="35,25"/>
		</screen>"""
		
	def __init__(self, session, args = 0):
		self.skin = DeviceLabelerScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)

		self["entryinfo"] = Label()
		self["menulist"] = deviceMenuList([])
		self["changeButton"] = Label(_("Edit"))
		self["changeButtonPixmap"] = Pixmap()

		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.changeLabel, _("Change label")),
			"cancel":	(self.closePlugin, _("Close plugin")),
		}, -1)

		self["menulist"].onSelectionChanged.append(self.selectionChanged)

		self.container = eConsoleAppContainer()
		self.container.appClosed.append(self.runFinished)
		self.container.dataAvail.append(self.dataAvail)

		self.initCommands()
	
	def initCommands(self):
		self.devicelist = []
		self.attributes = []
		self.label = []
		self.counter = 0
		self.cmdlist = []
		self.run = 0

		self.cmdlist.append("fdisk -l | grep /dev")
		
		self.container.execute(self.cmdlist[self.run])
		
	def dataAvail(self, outputstring):
		print "[DeviceLabeler - Data available] %s" %(outputstring)
		if outputstring.startswith("Filesystem volume name:"):
			labelline = outputstring.split(":")
			self.label.append(labelline[1].strip())
			self.attributes[self.counter].append(labelline[1].strip())
			self.counter = self.counter + 1
		else:
			lines = []
			lines = outputstring.split("\n")
			for i in range(len(lines)):
				if lines[i].startswith("/dev"):
					devices = lines[i].split()
					print "devices: ", devices
					self.cmdlist.append("tune2fs -l %s | grep volume" %(devices[0]))
					print "cmdlist: ", self.cmdlist
					if devices[2] == "*":
						#if we have boot set (* in devices[2]) we have to move the size value to devices[3]
						devices[3] = devices[4]
					devices[3] = devices[3].strip("+-")
					devsize = (float(devices[3])*1.024)/1000000
					if devsize < 1:
						devsize = devsize*1000
						devices[3] = str("%d MB" %(devsize))
					else:
						devices[3] = str("%d GB" %(devsize))
					self.attributes.append([devices[0], devices[3]])

	def runFinished(self, retval):
		print "[DeviceLabeler - Command executed] %d" %(retval)
		if retval != 0 and self.run>0:
			self.attributes[self.run-1].append("Error!")
			self.counter = self.counter+1
			
		self.run += 1
		if self.run < len(self.cmdlist):
			self.container.execute(self.cmdlist[self.run])
		else:
			self.run = 0
			self.cmdlist = []
			self.buildScreen()
		
	def buildScreen(self):
		print "attributes: ", self.attributes
		
		for i in range(len(self.attributes)):
			self.devicelist.append(deviceMenuListEntry(self.attributes[i][0], self.attributes[i][1], self.attributes[i][2]))
		self["menulist"].l.setList(self.devicelist)
		self["entryinfo"].setText("%d / %d" %(self["menulist"].getSelectedIndex()+1, self.counter))
	
	def selectionChanged(self):
		self["entryinfo"].setText("%d / %d" %(self["menulist"].getSelectedIndex()+1, self.counter))
		
	def changeLabel(self):
		self.session.openWithCallback(self.initCommands, DeviceLabelerEditScreen, self.attributes[self["menulist"].getSelectedIndex()])

	def closePlugin(self):
		self.container.appClosed.remove(self.runFinished)
		self.close()
			
class DeviceLabelerEditScreen(Screen,ConfigListScreen, HelpableScreen):
	skin = """
		<screen position="center,center" size="600,275" title="Device Labeler" >
			<widget itemHeight="25" name="config" position="0,40" size="600,25" scrollbarMode="showNever"/>
			<eLabel foregroundColor="#f0f0f0" font="Regular;18" position="275,211" size="200,25" text="Change label" transparent="1"/>
			<eLabel foregroundColor="#f0f0f0" font="Regular;18" position="275,236" size="200,25" text="Cancel" transparent="1"/>
			<ePixmap alphatest="on" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/DeviceLabeler/green.png" position="230,210" size="35,25"/>
			<ePixmap alphatest="on" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/DeviceLabeler/exit.png" position="230,235" size="35,25"/>
		</screen>"""
		
	def __init__(self, session, selectedEntry):
		self.skin = DeviceLabelerEditScreen.skin
		self.session = session
		self.selectedEntry = selectedEntry
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green": (self.doUpdate, _("Change label")),
		}, -1)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.close, _("Close edit screen")),
		}, -1)

		self.list = []
		ConfigListScreen.__init__(self, self.list)
		self.newlabel = NoSave(ConfigText(default = "", fixed_size = False))
		self.list.append(getConfigListEntry(_("New label:"), self.newlabel))
		self["config"].setList(self.list)		
		
		self.onShown.append(self.setWindowTitle)

	def setWindowTitle(self):
		self.setTitle(_("Edit label for: %s" %(self.selectedEntry[0])))
		
	def doUpdate(self):
		os.system("tune2fs -L %s %s" %(self.newlabel.value, self.selectedEntry[0]))
		self.close()
		
def Plugins(**kwargs):
    return [PluginDescriptor(name="Device Labeler", description=_("Plugin to change the label of a device"), where = [PluginDescriptor.WHERE_PLUGINMENU], icon="DeviceLabeler.png", fnc=main)]
